@@ -142,8 +142,11 @@ def load_plugins():
     for section in sections:
         conf = dict(config.items(section))
         conf['name'] = section
-        plugin = plugin_types[conf['plugin']]
-        components.append( plugin(conf) )
+        try:
+            plugin = plugin_types[conf['plugin']]
+            components.append( plugin(conf) )
+        except KeyError:
+            raise Exception("Plugin missing from config for section %r" % section)
 
     for section in config.get('general', 'order').split(', '):
         order.append(section)
@@ -211,6 +214,7 @@ def plugin_read_fd(config):
             # Prevent from blocking if a FIFO.
             self.fd = os.open(path.expanduser(filename), os.O_RDWR) 
             self.line_buf = ""
+            self.complete_line = ""
 
             polled_fds.append(self.fd)  
 
@@ -219,17 +223,19 @@ def plugin_read_fd(config):
                 t = os.read(self.fd, 4096)
                 if t:
                     self.line_buf += t
-                    self.line_buf = self.line_buf.splitlines()[-1] + '\n'
+                    self.complete_line = self.line_buf.splitlines()[-1]
+                    if len(self.line_buf.splitlines()) > 1 and self.line_buf.endswith('\n'):
+                        self.line_buf = '\n'.join(self.line_buf.splitlines()[-2:])  + '\n'
             except OSError, e:
                 # Is EAGAIN ever going to happen without O_NONBLOCK/O_NDELAY?
                 if e.errno != 11: # EAGAIN
                     raise
 
             # If longer than line_limit, truncate to limit and add elipsis.
-            if self.line_limit and len(self.line_buf) > self.line_limit:
-                self.line_buf  = self.line_buf[:self.line_limit] + '...\n'
+            if self.complete_line and self.line_limit != 0 and len(self.complete_line) > self.line_limit:
+                self.complete_line = self.complete_line[:self.line_limit] + '...'
 
-            return self.line_buf.strip()
+            return self.complete_line.strip()
 
     reader = LBNBFDReader(config['file'], int(config.get('line_limit', 0)))
     def updatep(now, last_update):
@@ -363,10 +369,12 @@ def plugin_battery(config):
             bat_color: Color of icon for when on battery power.
             high_color: Color for when battery has a good charge.
             low_color: Color for when batteyr has a low charge.
+            ac_name: The sysfs directory name for the AC adaptor.
+            battery_name: The sysfs directory name for the Battery.
     '''
     def do_battery():
-        battery_dir = '/sys/class/power_supply/CMB1/'
-        ac_dir = '/sys/class/power_supply/AC/'
+        battery_dir = '/sys/class/power_supply/%s/' % config['battery_name']
+        ac_dir = '/sys/class/power_supply/%s/' % config['ac_name']
 
         mAh_left = float(open(path.join(battery_dir, 'charge_now')).read().strip())
         mAh_max  = float(open(path.join(battery_dir, 'charge_full_design')).read().strip())
